// nvc.cpp : implementation file
//


#include "nvcc.h"
#include "nvc.h"
#ifndef WIN32
#include <pwd.h>
#endif
#include "nvcj.h"
#include "nvcfx.h"
#include "java.h"


using namespace std;


NvClient MainClient;	// Global Client app object

bool bRandInitialized = false;	// for intializing random generator
NvMutex ProtectRandInit;

#ifdef WIN32
// Main dll entry point for windows
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	if(ul_reason_for_call == DLL_PROCESS_DETACH)
		MainClient.ExitInstance();
		
	return 1;   // ok
}
#endif



NvClient* GetMain()
{
	return &MainClient;
}


void NVSTDCALL NvExitLib()
{
	MainClient.ExitInstance();
}


NVREQUEST NVSTDCALL NvOpenRequest(LPCSTR ConnectString)
{
	//initialize the random generator if not done
	NV_LOCKLOCK(&ProtectRandInit)
	if(!bRandInitialized)
	{
		srand((unsigned)time(NULL));
		bRandInitialized = true;
	}
	NV_UNLOCKLOCK(&ProtectRandInit)
	
	MainClient.InitInstance();
	string ProviStr;
	// The garbage collected is checked for each new request
//	MainClient.Garbage->RemoveExpired();

	NvRequest *Request = GetMain()->Requests.AddRequest();
	if(Request == NULL)
		return NULL;

	// This command object is used only for the parsing of parameters
	NvServerCommand ProviCommand;
	ProviCommand.ParseParameters(ConnectString);

	Request->m_sServer = ProviCommand.m_Parameters.GetValue("Server");
	// Check server address for load balancing
	if(Request->m_sServer.length())
	{
		string ServerPart = Request->m_sServer;
		string AddressHere = ServerPart;
		MakeUpper(AddressHere);
		string::size_type index = AddressHere.find("PROXY::");
		string Before = "";
		if(index != string::npos)
		{
			index = ServerPart.rfind("::");
			Before = ServerPart.substr(0, index+2);
			ServerPart = ServerPart.substr(index+2);
		}
		else
		{
			// Compatibiliy IPV6
			index = AddressHere.find("PROXY%%");
			if(index != string::npos)
			{
				index = ServerPart.rfind("%%");
				Before = ServerPart.substr(0, index+2);
				ServerPart = ServerPart.substr(index+2);
			}
		}
		index = ServerPart.rfind(":");
		string After = "";
		if(index != string::npos)
		{
			After = ServerPart.substr(index);
			ServerPart = ServerPart.substr(0, index);
		}
		if(ServerPart.find(";") != string::npos)
		{
			// There is several servers address in load balancing mode
			vector<string> Servers;
			while(ServerPart.length())
			{
				index = ServerPart.find(";");
				if(index == string::npos)
				{
					Servers.push_back(ServerPart);
					ServerPart = "";
				}
				else
				{
					Servers.push_back(ServerPart.substr(0, index));
					ServerPart = ServerPart.substr(index+1);
				}
			}
			int NumServers = (int)Servers.size();
			// We must generate a random number between 0 and NumServers-1
			int iServerIndex = rand() % NumServers;
			ServerPart = Servers[iServerIndex];
		}
		Request->m_sServer = Before + ServerPart + After;
	}

	Request->m_sAppli = ProviCommand.m_Parameters.GetValue("Application");
	Request->m_sSessionId = ProviCommand.m_Parameters.GetValue("Session");
	Request->m_sOpen = ProviCommand.m_Parameters.GetValue("Open");
	Request->m_sClose = ProviCommand.m_Parameters.GetValue("Close");
	Request->m_sCertificate = ProviCommand.m_Parameters.GetValue("Certificate");
	Request->m_sUser = ProviCommand.m_Parameters.GetValue("User");
	Request->m_sPassword = ProviCommand.m_Parameters.GetValue("Password");
	Request->bWithNewPassword = false;
	if(ProviCommand.m_Parameters.KeyExist("NV_NEW_PASSWORD"))
		Request->bWithNewPassword = true;
	Request->m_sNewPassword = ProviCommand.m_Parameters.GetValue("NV_NEW_PASSWORD");
	Request->bWithNewPasswordConfirm = false;
	if(Request->bWithNewPassword)
	{
		if(ProviCommand.m_Parameters.KeyExist("NV_NEW_PASSWORD_CONFIRM"))
			Request->bWithNewPasswordConfirm = true;
	}
	Request->m_sNewPasswordConfirm = ProviCommand.m_Parameters.GetValue("NV_NEW_PASSWORD_CONFIRM");
	Request->m_sTempDir = ProviCommand.m_Parameters.GetValue("TempDir");
	CleanString(Request->m_sTempDir);
	AddDirSuffix(Request->m_sTempDir);
	Request->m_iMaxMessMemSize = 1024 * 20;		// 20 Mbytes is the default
	if(ProviCommand.m_Parameters.KeyExist("MaxMsgSize"))
	{
		ProviStr = ProviCommand.m_Parameters.GetValue("MsgSize");
		int MaxMessMemSize = atoi(ProviStr.c_str());
		Request->m_iMaxMessMemSize = MaxMessMemSize;
		if(Request->m_iMaxMessMemSize < 10)
			Request->m_iMaxMessMemSize = 10;
	}
	Request->m_iMaxMessMemSize *= 1024; // From kilobytes to bytes
	Request->m_sSocketLog = ProviCommand.m_Parameters.GetValue("SocketLog");

	CleanString(Request->m_sServer);
	CleanString(Request->m_sAppli);
	CleanString(Request->m_sSessionId);
	CleanString(Request->m_sUser);
	CleanString(Request->m_sPassword);
	CleanString(Request->m_sNewPassword);
	CleanString(Request->m_sNewPasswordConfirm);
	Request->m_sHash = Request->m_sAppli + Request->m_sUser + Request->m_sPassword;
	Request->m_sHash = MD5_string(Request->m_sHash);
	MakeUpper(Request->m_sHash);

	if(!Request->m_sServer.length())
		Request->m_sServer = "127.0.0.1";

	string ServerHere = Request->m_sServer;
	MakeUpper(ServerHere);
	if(ServerHere.find("NV.") == 0)
		Request->m_sServer = Request->m_sServer.substr(3);

	ProviStr = ProviCommand.m_Parameters.GetValue("Autoclose");
	CleanString(ProviStr);
	MakeUpper(ProviStr);
	if(ProviStr == "YES")
		Request->m_bAutoClose = true;
	else
		Request->m_bAutoClose = false;

	ProviStr = ProviCommand.m_Parameters.GetValue("Ssl");
	CleanString(ProviStr);
	MakeUpper(ProviStr);
	if(ProviStr == "YES")
		Request->b_UseSSL = true;
	else
		Request->b_UseSSL = false;

	ProviStr = ProviCommand.m_Parameters.GetValue("Sso");
	CleanString(ProviStr);
	MakeUpper(ProviStr);
	if(ProviStr == "YES")
		Request->b_UseSSO = true;
	else
		Request->b_UseSSO = false;

	Request->SSOPrincipal = ProviCommand.m_Parameters.GetValue("SsoPrincipal");;
	CleanString(Request->SSOPrincipal, true);

	ProviStr = ProviCommand.m_Parameters.GetValue("Unicode");
	CleanString(ProviStr);
	MakeUpper(ProviStr);
	if(ProviStr == "YES")
		Request->b_ClientUnicode = true;
	else
		Request->b_ClientUnicode = false;

	if(ProviCommand.m_Parameters.KeyExist("SessionTimeOut"))
	{
		ProviStr = ProviCommand.m_Parameters.GetValue("SessionTimeOut");
		int SessionTimeOut = atoi(ProviStr.c_str());
		Request->m_iSessionTimeOut = SessionTimeOut;
		if(Request->m_iSessionTimeOut < 0)
			Request->m_iSessionTimeOut = 0;
	}

	if(ProviCommand.m_Parameters.KeyExist("ConnectionTimeOut"))
	{
		ProviStr = ProviCommand.m_Parameters.GetValue("ConnectionTimeOut");
		int ConnectionTimeOut = atoi(ProviStr.c_str());
		Request->m_iServerConnectionTimeOut = ConnectionTimeOut;
		if(Request->m_iServerConnectionTimeOut < 0)
			Request->m_iServerConnectionTimeOut = 0;
	}

	return ((void *)Request);
}

void NVSTDCALL NvCloseRequest(NVREQUEST Request)
{
	MainClient.InitInstance();
	NvRequest *RequestHere = (NvRequest *)Request;
	GetMain()->Requests.RemoveRequest(RequestHere);
}

// Main client command function
// Returns 1 if OK, 0 if a command error occurs and -1 if the request handle is bad
int NVSTDCALL NvCommand(NVREQUEST Request, LPCSTR Command, LPSTR Buffer, int BufferSize, int *DataSize)
{
	return NvCommand2((void *)Request, Command, Buffer, BufferSize, DataSize, ENCODING_NONE);
}


int NvCommand2(void* Request, const char* Command, char* Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	MainClient.InitInstance();
	NvRequest *RequestHere = (NvRequest *)Request;

	if(RequestHere == NULL)
		return -1;

	// next line is commented for performance reason
	// Put a parameter
//	if(!GetMain()->Requests.CheckRequest(RequestHere))
//		return -1;
	bool bValidRequest = true;
	bool bForTestOnly;
	try
	{
		// Should crash if the request doesn't exist
		bForTestOnly = RequestHere->b_UseSSL;
	}
	catch(...)
	{
		bValidRequest = false;
	}
	if(!bValidRequest)
		return -1;

	int RetVal = 1;
	try
	{
		if(!RequestHere->NvCommand(Command, Buffer, BufferSize, DataSize, iEncoding))
			RetVal = 0;
	}
	catch(...)
	{
		RetVal = -1;
	}
	return RetVal;
}



/////////////////////////////////////////////////////////////////////////////
// NvClient construction

NvClient::NvClient()
{
	Initialized = false;
	MachineName = "";
	MachineUser = "";
	Garbage = NULL;
	JavaRequests.m_bUseChecking = false;
}


NvClient::~NvClient()
{
	ExitInstance();
}


bool NvClient::InitInstance() 
{
	if(Initialized)
		return true;

#ifdef WIN32
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;

	// initialize window sockets
	wVersionRequested = MAKEWORD( 1, 1 );

	err = WSAStartup(wVersionRequested, &wsaData);
	if ( err != 0 )
	{
		WSACleanup();
		return false;
	}
#endif

	// Get machine name and user
	char buffer[1024];
	memset(buffer,0,sizeof(buffer));
#ifdef WIN32
	unsigned long Size;
/*	Size = sizeof(buffer);
	GetUserName(buffer, &Size);
	MachineUser = buffer;
	memset(buffer,0,sizeof(buffer));
	Size = sizeof(buffer);
	GetComputerName(buffer, &Size);
	MachineName = buffer;
*/
	wchar_t wbuf[1024]; 
	Size = sizeof(wbuf);
	if(GetUserNameW(wbuf, &Size))
	{
		WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, buffer, 1024, NULL, NULL);
		MachineUser = buffer;
	}
	Size = sizeof(wbuf);
	if(GetComputerNameW(wbuf, &Size))
	{
		WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, buffer, 1024, NULL, NULL);
		MachineName = buffer;
	}
#else
	struct passwd *Ident = getpwuid(getuid());
	strcpy(buffer, Ident->pw_name);
	MachineUser = buffer;
	gethostname(buffer, sizeof(buffer));
	MachineName = buffer;
#endif

	Initialized = true;

	return true;
}


void NvClient::ExitInstance() 
{
	if(Initialized)
	{
		JavaRequests.Clear();
		CfRequests.Clear();
		Requests.Clear();
//		Garbage.Clear();
#ifdef WIN32
		WSACleanup();
#endif
		Initialized = false;
	}
}






// Java interface functions
JNIEXPORT jlong JNICALL Java_com_nirvasoft_nirva_nvcj_Int_1OpenRequest(JNIEnv *env, jobject jobj, jstring ConnectString)
{
	const char *str = env->GetStringUTFChars(ConnectString, 0);
	string ConnectStringHere = str;
	// Force unicode for java
	ConnectStringHere += " Unicode=\"YES\"";
	env->ReleaseStringUTFChars(ConnectString, str);
	NvcJavaInt* JavaInt = GetMain()->JavaRequests.AddRequest();
	if(JavaInt == NULL)
		return 0;
	if(!JavaInt->OpenRequest(ConnectStringHere.c_str()))
	{
		GetMain()->JavaRequests.RemoveRequest(JavaInt);
		return 0;
	}
	return ((jlong)(JavaInt));
}



JNIEXPORT void JNICALL Java_com_nirvasoft_nirva_nvcj_Int_1CloseRequest(JNIEnv *env, jobject jobj, jlong RequestHandle)
{
	NvcJavaInt* JavaInt = (NvcJavaInt*)RequestHandle;
	if(JavaInt == NULL)
		return;

	GetMain()->JavaRequests.RemoveRequest(JavaInt);
}


JNIEXPORT jint JNICALL Java_com_nirvasoft_nirva_nvcj_Int_1Command(JNIEnv *env, jobject jobj, jlong RequestHandle, jstring CommandString)
{
	NvcJavaInt* JavaInt = (NvcJavaInt*)RequestHandle;
	if(JavaInt == NULL)
		return 0;
	if(!GetMain()->JavaRequests.CheckRequest(JavaInt))
		return 0;

	const char *str = env->GetStringUTFChars(CommandString, 0);
	string CommandStringHere = str;
	env->ReleaseStringUTFChars(CommandString, str);

	jint RetVal = JavaInt->Command(CommandStringHere.c_str());
	GetMain()->JavaRequests.FreeRequest(JavaInt);

	return RetVal;
}

JNIEXPORT jstring JNICALL Java_com_nirvasoft_nirva_nvcj_Int_1GetResult(JNIEnv *env, jobject jobj, jlong RequestHandle)
{
	string Result = "";

	NvcJavaInt* JavaInt = (NvcJavaInt*)RequestHandle;
	if(JavaInt == NULL)
		return env->NewStringUTF(Result.c_str());
	if(!GetMain()->JavaRequests.CheckRequest(JavaInt))
		return env->NewStringUTF(Result.c_str());

	Result = JavaInt->LastResult;
	GetMain()->JavaRequests.FreeRequest(JavaInt);

	return env->NewStringUTF(Result.c_str());
}




// Java interface functions for cold fusion
JNIEXPORT jlong JNICALL Java_nvcfx_Int_1OpenRequest(JNIEnv *env, jobject jobj, jstring ConnectString)
{
	const char *str = env->GetStringUTFChars(ConnectString, 0);
	string ConnectStringHere = str;
	// Force unicode for java
	ConnectStringHere += " Unicode=\"YES\"";
	env->ReleaseStringUTFChars(ConnectString, str);
	NvcJavaInt* JavaInt = GetMain()->CfRequests.AddRequest(3600);	// time out to 3600 seconds for cold fusion
	if(JavaInt == NULL)
		return 0;
	if(!JavaInt->OpenRequest(ConnectStringHere.c_str()))
	{
		GetMain()->JavaRequests.RemoveRequest(JavaInt);
		return 0;
	}
	return ((jlong)(JavaInt));
}



JNIEXPORT void JNICALL Java_nvcfx_Int_1CloseRequest(JNIEnv *env, jobject jobj, jlong RequestHandle)
{
	NvcJavaInt* JavaInt = (NvcJavaInt*)RequestHandle;
	if(JavaInt == NULL)
		return;

	GetMain()->CfRequests.RemoveRequest(JavaInt);
}


JNIEXPORT jint JNICALL Java_nvcfx_Int_1Command(JNIEnv *env, jobject jobj, jlong RequestHandle, jstring CommandString)
{
	NvcJavaInt* JavaInt = (NvcJavaInt*)RequestHandle;
	if(JavaInt == NULL)
		return 0;
	if(!GetMain()->CfRequests.CheckRequest(JavaInt))
		return 0;

	const char *str = env->GetStringUTFChars(CommandString, 0);
	string CommandStringHere = str;
	env->ReleaseStringUTFChars(CommandString, str);

	jint RetVal = JavaInt->Command(CommandStringHere.c_str());
	GetMain()->CfRequests.FreeRequest(JavaInt);

	return RetVal;
}

JNIEXPORT jstring JNICALL Java_nvcfx_Int_1GetResult(JNIEnv *env, jobject jobj, jlong RequestHandle)
{
	string Result = "";

	NvcJavaInt* JavaInt = (NvcJavaInt*)RequestHandle;
	if(JavaInt == NULL)
		return env->NewStringUTF(Result.c_str());
	if(!GetMain()->CfRequests.CheckRequest(JavaInt))
		return env->NewStringUTF(Result.c_str());

	Result = JavaInt->LastResult;
	GetMain()->CfRequests.FreeRequest(JavaInt);

	return env->NewStringUTF(Result.c_str());
}



// Exported functions for .net
// We have special functions because of encoding that is unicode in dotnet and is UTF-8 on nirva side
#ifdef WIN32
#ifdef __cplusplus
extern "C" {
#endif

unsigned long long NVSTDCALL NvOpenRequestW(LPCWSTR ConnectString);
void NVSTDCALL NvCloseRequestW(unsigned long long Request);
int NVSTDCALL NvCommandW(unsigned long long Request, LPCWSTR Command, LPWSTR Buffer, int BufferSize, int *DataSize);

#ifdef __cplusplus
}
#endif



unsigned long long NVSTDCALL NvOpenRequestW(LPCWSTR ConnectString)
{
	int Size = WideCharToMultiByte(CP_UTF8, 0, ConnectString, -1, NULL, 0, NULL, NULL);
	LPSTR Chars = (LPSTR)malloc(Size);
	if(Chars == NULL)
		return 0;
	WideCharToMultiByte(CP_UTF8, 0, ConnectString, -1, Chars, Size, NULL, NULL);
	unsigned long long RetVal = 0;
	try
	{
		RetVal = (unsigned long long)NvOpenRequest(Chars);
	}
	catch(...)
	{
	}

	free(Chars);
	return RetVal;
}

void NVSTDCALL NvCloseRequestW(unsigned long long Request)
{
	NvCloseRequest((void *)Request);
}

int NVSTDCALL NvCommandW(unsigned long long Request, LPCWSTR Command, LPWSTR Buffer, int BufferSize, int *DataSize)
{
	int CommandSize = WideCharToMultiByte(CP_UTF8, 0, Command, -1, NULL, 0, NULL, NULL);
	
	LPSTR CommandChars = (LPSTR)malloc(CommandSize);
	if(CommandChars == NULL)
		return -1;
	
	WideCharToMultiByte(CP_UTF8, 0, Command, -1, CommandChars, CommandSize, NULL, NULL);
	int RetVal = 0;
	LPSTR ResultChars = (LPSTR)malloc(BufferSize);
	try
	{
		RetVal = NvCommand((void *)Request, CommandChars, ResultChars, BufferSize, DataSize);
	}
	catch(...)
	{
	}
	free(CommandChars);
	MultiByteToWideChar(CP_UTF8, 0, ResultChars, -1, Buffer, BufferSize);
	free(ResultChars);
	return RetVal;
}


#endif //WIN32
