//*****************************************************************************
// File: nvd.cpp
// Description: Nirva nvd utility
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifdef WIN32
#pragma warning( disable : 4996 )
#endif

#ifdef WIN32
#define _CRT_NONSTD_NO_DEPRECATE
#endif
#ifndef WIN32
#include <pthread.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include <sys/types.h>
#ifdef WIN32
#include <windows.h>
#include <io.h>
#else
#include <unistd.h>
#include <ctype.h>
#endif
#include "../nvc/nvc.h"
#include "../nvs/task.h"
#include "util.h"


using namespace std;


#define NV_VERSION	"1.00"


// Local functions
int ModeConsole(int argc, char* argv[]);
int GetCommandLineOption(int argc, char* argv[], char option, int more);
int GetParametersFromCommandLine(int argc, char* argv[]);
void DisplayHelp();
bool CheckError(NVREQUEST Request, int Result);
void DoCommand(NVREQUEST Request, std::string Session, std::string Command, bool NoClose);

class ThreadClient : public NvTask
{
// Attributes
public:
	NVREQUEST Request;
	string Session;
	string Command;
	bool NoClose;

	// Overridable callbacks
protected:
	virtual int Main();
	virtual void OnException(std::string sInfo);
	virtual bool OnInit();
	virtual void OnCleanup();

public:

	ThreadClient();
};


// Global variables
vector<string> CommandParameters;		// Command parameters
char OptionBuffer[2048];				// Contains option value
string SessionId;
ThreadClient Client;

int main(int argc, char* argv[])
{
	char Buffer[2048];		// General buffer
	
	SessionId = "";

	if(argc == 1)
	{
		DisplayHelp();
		return 0;
	}
	if((argc == 2) && (strlen(argv[1]) >= 2))
	{
		strcpy(Buffer, argv[1]);
		if((Buffer[0] == '-') && (Buffer[1] == 'h'))
		{
			DisplayHelp();
			return 0;
		}
	}
	
	// Check for mode
	string Mode = "CONSOLE";
	int Error = GetCommandLineOption(argc, argv, 'm', 1);
	if(Error == 0)
	{
		Mode = OptionBuffer;
		MakeUpper(Mode);
		CleanString(Mode);
	}

	if(Mode == "CONSOLE")
	{
		return ModeConsole(argc, argv);
	}

	DisplayHelp();
	return 0;
}

int ModeConsole(int argc, char* argv[])
{
	// Send a SESSION:ATTACH_CONSOLE to nirva
	char CommandBuffer[4096];
	char OutputBuffer[4096];
	string Session = "";
	string Port = "";
	string Server = "127.0.0.1:";
	string Command = "";
	string Procedure = "";
	string User = "";
	string Password = "";
	string Application = "";
	string OpenProc = "";
	string CloseProc = "";
	bool bNoClose = false;
	int Level = 1;

	int Error = GetCommandLineOption(argc, argv, 's', 1);
	if(Error == 0)
	{
		Session = OptionBuffer;
		MakeUpper(Session);
		CleanString(Session);
	}

	Error = GetCommandLineOption(argc, argv, 'z', 1);
	if(Error == 0)
		Command = OptionBuffer;

	Error = GetCommandLineOption(argc, argv, 'r', 1);
	if(Error == 0)
		Procedure = OptionBuffer;
	
	Error = GetCommandLineOption(argc, argv, 'u', 1);
	if(Error == 0)
		User = OptionBuffer;

	Error = GetCommandLineOption(argc, argv, 'w', 1);
	if(Error == 0)
		Password = OptionBuffer;

	Error = GetCommandLineOption(argc, argv, 'a', 1);
	if(Error == 0)
		Application = OptionBuffer;

	Error = GetCommandLineOption(argc, argv, 'o', 1);
	if(Error == 0)
		OpenProc = OptionBuffer;
	string::size_type index = OpenProc.find('!');
	if(index != string::npos)
	{
		CloseProc = OpenProc.substr(index+1);
		OpenProc = OpenProc.substr(0, index);
	}

	Error = GetCommandLineOption(argc, argv, 'c', 0);
	if(Error == 0)
		bNoClose = true;

	Error = GetCommandLineOption(argc, argv, 'l', 1);
	if(Error == 0)
	{
		Level = atoi(OptionBuffer);
		if(Level < 1)
			Level = 1;
	}

	if(!Command.length())
	{
		if(Procedure.length())
		{
			Command = "NV_PROC=|";
			Command += Procedure;
			Command += "|";
		}
	}

	Error = GetCommandLineOption(argc, argv, 'p', 1);
	if(Error == 0)
	{
		Port = OptionBuffer;
		MakeUpper(Port);
		CleanString(Port);
	}
	if(!Port.length())
		Port = "1081";

	Server += Port;

	string ConnectString = "Server=\"";
	ConnectString += Server;
	ConnectString += "\" ConnectionTimeOut=\"3\"";
	
	string Type = "";
	if(Session.length())
	{
		MakeUpper(Session);
		CleanString(Session);
		index = Session.find("NEXT:");
		if(index != string::npos)
		{
			Type = Session.substr(index+5);
			Session = "";
			bNoClose = true; // The session closes it itself
		}
		index = Session.find("NEXT");
		if(index != string::npos)
		{
			// When  just -s next is given this is to attach to the next session whatever its type
			Type = "ANY";
			Session = "";
			bNoClose = true; // The session closes it itself
		}
	}
	if(Session.length())
	{
		bNoClose = true;
		ConnectString += " Session=\"";
		ConnectString += Session;
		ConnectString += "\"";
	}
	if(Application.length())
	{
		ConnectString += " Application=\"";
		ConnectString += Application;
		ConnectString += "\"";
	}
	else if(!Type.length())
	{
		if(User.length())
		{
			ConnectString += " User=\"";
			ConnectString += User;
			ConnectString += "\"";
		}
		if(Password.length())
		{
			ConnectString += " Password=\"";
			ConnectString += Password;
			ConnectString += "\"";
		}
		if(OpenProc.length())
		{
			ConnectString += " Open=\"";
			ConnectString += OpenProc;
			ConnectString += "\"";
		}
		if(CloseProc.length())
		{
			ConnectString += " Close=\"";
			ConnectString += CloseProc;
			ConnectString += "\"";
		}
	}
	
	NVREQUEST Request = NvOpenRequest(ConnectString.c_str());
	if(Request == NULL)
	{
		printf("Error opening request\n");
		return -1;
	}
	// Sends the Nirva command
	int CurrentProcess = getpid();
	sprintf(CommandBuffer, "NV_CMD=|SESSION:ATTACH_CONSOLE| PID=|%d| TYPE=|%s| LEVEL=|%d|", CurrentProcess, Type.c_str(), Level);
	int Result = NvCommand(Request, CommandBuffer, OutputBuffer, sizeof(OutputBuffer), NULL);
	if(!CheckError(Request, Result))  // Check the result
	{
		if(!bNoClose)
			NvCommand(Request, "NV_CMD=|SESSION:CLOSE|", NULL, 0, NULL);
		NvCloseRequest(Request);
		return -1;
	}

	printf("Waiting for a session to be attached...\n");
	
	// Execute the eventual command (done in separate thread)
	if(!Type.length())
		DoCommand(Request, Session, Command, bNoClose);

#ifdef WIN32
	// The result buffer contains the Pipe Handle
	string sPipeHandle = OutputBuffer;
	HANDLE hPipeHandle = (HANDLE)(NvAtoi64(sPipeHandle.c_str()));

	DWORD dwRead; 
	if(hPipeHandle != NULL)
	{
		while(ReadFile(hPipeHandle, OutputBuffer, sizeof(OutputBuffer)-1, &dwRead, NULL))
			_write(1, OutputBuffer, dwRead);

		CloseHandle(hPipeHandle);
	}
#else
	// The result buffer contains the fifo file name
	string sFifoFileName = OutputBuffer;
	int hPipeHandle = open(sFifoFileName.c_str(), O_RDONLY);

	int NumRead;
	if(hPipeHandle > 0)
	{
		while(1)
		{
			NumRead = read(hPipeHandle, OutputBuffer, sizeof(OutputBuffer)-1);
			if(NumRead <=0)
				break;
			write(1, OutputBuffer, NumRead);
		}
		close(hPipeHandle);
	}
#endif
	
	printf("\n\nConsole has been closed by nirva\n");

	while(Client.IsThreadRunning())
		SLEEP(10);
	return 0;
}

void DoCommand(NVREQUEST Request, string Session, string Command, bool NoClose)
{
	if(Session.length() && !Command.length())
	{
		// We don't close the session and we don't send any command 
		NvCloseRequest(Request);
		return;
	}

	Client.Request = Request;
	Client.Session = Session;
	Client.Command = Command;
	Client.NoClose = NoClose;
	Client.Run();
}


bool CheckError(NVREQUEST Request, int Result)
{
	if(Result == -1)
	{
		printf("Invalid request\n");
		return false;
	}
	if(Result == 0)
	{
		// Error
		printf("Nirva command error\n");
		char ErrorString[256];
		NvCommand(Request, "NV_CMD=|LOCAL:REQUEST:ERROR_INFO| ERROR_STRING=|SERVICE|", ErrorString, sizeof(ErrorString), NULL);
		printf("\tError service : %s\n", ErrorString);
		NvCommand(Request, "NV_CMD=|LOCAL:REQUEST:ERROR_INFO| ERROR_STRING=|CLASS|", ErrorString, sizeof(ErrorString), NULL);
		printf("\tError class : %s\n", ErrorString);
		NvCommand(Request, "NV_CMD=|LOCAL:REQUEST:ERROR_INFO| ERROR_STRING=|CODE|", ErrorString, sizeof(ErrorString), NULL);
		printf("\tError code : %s\n", ErrorString);
		NvCommand(Request, "NV_CMD=|LOCAL:REQUEST:ERROR_INFO| ERROR_STRING=|DESCRIPTION|", ErrorString, sizeof(ErrorString), NULL);
		printf("\tError description : %s\n", ErrorString);
		NvCommand(Request, "NV_CMD=|LOCAL:REQUEST:ERROR_INFO| ERROR_STRING=|INFO|", ErrorString, sizeof(ErrorString), NULL);
		printf("\tError info : %s\n", ErrorString);
		return false;
	}

	return true;
}


int GetCommandLineOption(int argc, char* argv[], char option, int more)
{
	char Buffer[2048];		// General buffer

	int i;
	for(i=1; i<argc; i++)
	{
		strcpy(Buffer, argv[i]);
		if(Buffer[0] != '-')
			continue;
		if(strlen(Buffer) <= 1)
			continue;
		if(Buffer[1] != option)
			continue;
		
		// we found the option
		if(!more)			// if nothing more to get
			return 0;		// we leave

		i++;
		if(i == argc)		// error extension not found
			return 1;

		strcpy(OptionBuffer, argv[i]);
		if(OptionBuffer[0] == '-')		// error extension not found
			return 1;

		CleanString(OptionBuffer);

		return 0;		// we got the extension into Buffer
	}

	return 1;		// we didn't find the option
}



// Builds the list of parameters
int GetParametersFromCommandLine(int argc, char* argv[])
{
	char Buffer[1024];		// General buffer

	strcpy(Buffer, "");
	int i;
	for(i=1; i<argc; i++)
	{
		if(argv[i][0] != '-')
			break;
		if(argv[i][1] == 'c')	// test no close option
			continue;
		if(argv[i][1] == 't')	// test no stop on error option
			continue;
		if(argv[i][1] == 'n')	// test unicode option
			continue;
		i++;
		if(i == argc)		// error extension not found
			return 1;

		if(argv[i][0] == '-')
			i--;
	}

	for(; i<argc; i++)
	{
		strcpy(Buffer, argv[i]);
		// discard first and end double quotes
		if(strlen(Buffer) && (Buffer[0] == '\"'))
			strcpy(Buffer, argv[i]+1);
		if(strlen(Buffer) && (Buffer[strlen(Buffer)-1] == '\"'))
			Buffer[strlen(Buffer)-1] = '\0';

		CleanString(Buffer);

		if(!strlen(Buffer))
			continue;

		CommandParameters.push_back(Buffer);

	}


	return 0;		// al is ok
}

	

void DisplayHelp()
{
	printf("\n\nUtility for debugging Nirva sessions");
	printf("\nversion : ");
	printf(NV_VERSION);
	printf("\nCopyright NIRVA SYSTEMS 2009");
	printf("\n\n\nusage: nvd [options]");
	printf("\n\noptions:");
	printf("\n          -s <session ID>");
	printf("\n             Nirva session to attach. If not provided, a new session is created.");
	printf("\n             Instead of a session ID a value \"next:type\" can be given. At this");
	printf("\n             time nvd connects to the next session of the given type. Type can");
	printf("\n             be one of: any, client, web, webs, xml, soap, transaction,");
	printf("\n             scheduler, named, listener, thread, internal, mq.");
	printf("\n             \"-s next\" is the same as \"-s next:any\"");
	printf("\n          -l <debug level>");
	printf("\n             Debug level. This can be 1, 2 or 3.");
	printf("\n             The default is 1.");
	printf("\n          -p <NIRVA http port>");
	printf("\n             This is the TCP/IP port of the NIRVA Http local server.");
	printf("\n             The default is 1081.");
	printf("\n          -a <application name>");
	printf("\n             Name of the NIRVA application to connect.");
	printf("\n             This parameter has meaning only when a session ID is not given.");
	printf("\n             The default is nvdef. If \"-s next\" is used and -a parameter is");
	printf("\n             not given nvd will connect the next given type session whatever the");
	printf("\n             application.");
	printf("\n          -u <user name>");
	printf("\n             Application user name.");
	printf("\n             This parameter has meaning only when -s parameter is not used.");
	printf("\n             The default is nvdef.");
	printf("\n          -w <user password>");
	printf("\n             User password.");
	printf("\n             This parameter has meaning only when -s parameter is not used.");
	printf("\n          -o <open procedure!close procedure>");
	printf("\n             Name of the session open and close procedures. The open and close");
	printf("\n             procedure names are separated by the ! character. If one is omitted");
	printf("\n             or if the -o parameter is not provided NIRVA uses session_open and");
	printf("\n             session_close as default names.");
	printf("\n             This parameter has meaning only when -s parameter is not used.");
	printf("\n          -r <procedure>");
	printf("\n             Name of Nirva procedure.");
	printf("\n             If this parameter is provided, nvd requests nirva to execute the");
	printf("\n             given procedure.");
	printf("\n             This parameter is not used when the next session is to be attached.");
	printf("\n          -z <nirva command>");
	printf("\n             Nirva command.");
	printf("\n             If this parameter is provided, nvd requests nirva to execute the");
	printf("\n             given command. This parameter has priority on the -r parameter (-r");
	printf("\n             is then ignored).");
	printf("\n             This parameter is not used when the next session is to be attached.");
	printf("\n          -c");
	printf("\n             No close option. If this option is given, nvd doesn't close the");
	printf("\n             NIRVA session when exiting.");
	printf("\n             The default is to close the session.");
	printf("\n             This parameter has meaning only when -s parameter is not used.");
	printf("\n          -h");
	printf("\n             Display help.");
	printf("\n\n");
}

ThreadClient::ThreadClient()
{
	Request = NULL;
	Session = "";
	Command = "";
	NoClose = false;
}

int ThreadClient::Main()
{
	NvCommand(Request, Command.c_str(), NULL, 0, NULL);
	return 0;
}

bool ThreadClient::OnInit()
{
	NvTask::OnInit();	// call the base class
	return true;
}

void ThreadClient::OnCleanup()
{
	if(!NoClose)
		NvCommand(Request, "NV_CMD=|SESSION:CLOSE|", NULL, 0, NULL);

	NvCloseRequest(Request);
	NvTask::OnCleanup();	// call the base class
}

void ThreadClient::OnException(string sInfo)
{
}
