// ***************************************************************************************
// File: ejava.h
// Description: interface to java
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _EJAVA_H
#define _EJAVA_H

#ifndef NVSTDCALL
#ifdef WIN32
#define NVSTDCALL __stdcall
#else
#define NVSTDCALL
#endif
#endif

#define NVJAVAVM void*

#define JAVAENV_TYPE_APPLICATION		"APPLICATION"
#define JAVAENV_TYPE_SERVICE			"SERVICE"
#define JAVAENV_TYPE_WEBSERVICE			"WSERVICE"
#define JAVAENV_TYPE_SYSTEM				"SYSTEM"

// log entry type
#define NV_LOG_ALL				-1
#define NV_LOG_INFO				0
#define NV_LOG_WARNING			1
#define NV_LOG_ERROR			2
#define NV_LOG_UNKNOWNTYPE		3


// log functions
typedef void (*JAVALOG)(const char *Message, const char *Extra, int Type, int Level, const char *Identifier);
typedef int (*JAVAGETLOGLEVEL)();

// java library entry points
typedef int (*JAVAINIT)(const char* NirvaDir, char* MinMemSize, char* MaxMemSize, const char* JavaHomeDir, const char* JvmPath, const char* JvmParam, const char* ExtraParam, JAVALOG syslog, JAVAGETLOGLEVEL getloglevel);
typedef void (*JAVAFREE)();
typedef int (*JAVARUNMETHOD)(const char* MethodName, void *pCommand);
typedef int (*JAVADETACHTHREAD)();
typedef int (*JAVAATTACHTHREAD)();
typedef void (*JAVAUNLOADENV)(const char* Type, const char* Name);


// Exported functions
extern "C"
{
int NvInitJava(const char* NirvaDir, char* MinMemSize, char* MaxMemSize, const char* JavaHomeDir, const char* JvmPath, const char* JvmParam, const char* ExtraParam, JAVALOG syslog, JAVAGETLOGLEVEL getloglevel);
void NvFreeJava();
int NvRunMethod(const char* MethodName, void *pCommand);
int NvDetachThread();
int NvAttachThread();
void NvUnloadEnv(const char* Type, const char* Name);
}


#endif //_EJAVA_H
