// ***************************************************************************************
// File: elc.h
// Description: interface to nirva license management
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: This file contains the declaration of the exported functions of
//           the nvlc library. The nvlc library contains proprietary code for Nirva
//           license management. The nvlc library is not open source.
// ***************************************************************************************

#ifndef _ELC_H
#define _ELC_H

#ifndef NVSTDCALL
#ifdef WIN32
#define NVSTDCALL __stdcall
#else
#define NVSTDCALL
#endif
#endif

#include <string>


typedef struct {
				char m_sService[256];
				char m_sServiceId[50];
				char m_sKey[256];
				char m_sValue[256];
				char m_sDate[20];
			  } NvlKey;

#define NVLCIDENT void*

// nvlc library entry points
typedef NVLCIDENT (*LC_OPEN)();
typedef void (*LC_CLOSE)(NVLCIDENT License);
typedef int (*INITLCLIB)(const char* NirvaPath);
typedef void (*EXITLCLIB)();
typedef void (*LC_SETLICENSEFILE)(NVLCIDENT License, const char* FileName);
typedef bool (*LC_SETKEY)(NVLCIDENT License, const char* MachineId, const char* Service, const char* ServiceId, const char* KeyName, const char* KeyValue, const char* KeyDate);
typedef bool (*LC_CHECKKEY)(NVLCIDENT License, const char* Service, const char* ServiceId, const char* KeyName, char* Value, int ValueSize, bool VerifyDate);
typedef bool (*LC_REMOVEKEY)(NVLCIDENT License, const char* Service, const char* ServiceId, const char* KeyName);
typedef bool (*LC_IMPORT)(NVLCIDENT License, const char* FileName);
typedef bool (*LC_EXPORT)(NVLCIDENT License, const char* FileName, const char* Service);
typedef int (*LC_GETNUMKEY)(NVLCIDENT License);
typedef bool (*LC_GETKEYBYINDEX)(NVLCIDENT License, int Index, NvlKey* Key);
typedef bool (*LC_GETKEYBYNAME)(NVLCIDENT License, const char* Service, const char* ServiceId, const char* KeyName, NvlKey* Key);
typedef bool (*LC_LOAD)(NVLCIDENT License);
typedef bool (*LC_SAVE)(NVLCIDENT License);
typedef void (*LC_GETMACHINEKEY)(NVLCIDENT License, char* MachineKey);
typedef void (*LC_SETMACHINEKEY)(NVLCIDENT License, const char* MachineKey);


// Exported functions
extern "C"
{
NVLCIDENT NvLcOpen();
void NvLcClose(NVLCIDENT License);
void NvLcSetLicenseFile(NVLCIDENT License, const char* FileName);
bool NvLcSetKey(NVLCIDENT License, const char* MachineId, const char* Service, const char* ServiceId, const char* KeyName, const char* KeyValue, const char* KeyDate);
bool NvLcCheckKey(NVLCIDENT License, const char* Service, const char* ServiceId, const char* KeyName, char* Value, int ValueSize, bool VerifyDate);
bool NvLcRemoveKey(NVLCIDENT License, const char* Service, const char* ServiceId, const char* KeyName);
bool NvLcImport(NVLCIDENT License, const char* FileName);
bool NvLcExport(NVLCIDENT License, const char* FileName, const char* Service);
int NvLcGetNumKey(NVLCIDENT License);
bool NvLcGetKeyByIndex(NVLCIDENT License, int Index, NvlKey* Key);
bool NvLcGetKeyByName(NVLCIDENT License, const char* Service, const char* ServiceId, const char* KeyName, NvlKey* Key);
bool NvLcLoad(NVLCIDENT License);
bool NvLcSave(NVLCIDENT License);
void NvLcGetMachineKey(NVLCIDENT License, char* MachineKey);
void NvLcSetMachineKey(NVLCIDENT License, const char* MachineKey);
int InitLcLib(const char* NirvaPath);
void ExitLcLib();
}


#endif //_ELC_H
