// ***************************************************************************************
// File: compress.h
// Description: declarations of the huffmann compression functions and classes
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments:
// ***************************************************************************************

#ifndef _COMPRESS_H
#define _COMPRESS_H

#include "task.h"
// Classes declared in this file
class CHufComp;


/**************************************************************************/
/**************************************************************************/

#define         LZH_NBUFS        4096            /* Buffer size*/
#define         LZH_LOOKA          10            /* Lookahead buffer size */
#define         LZH_THRES           2            /* Threshold*/
#define         LZH_NCHAR       (256 -LZH_THRES +LZH_LOOKA)
#define         LZH_TABLE       (LZH_NCHAR *2 -1)
#define         LZH_ROOTP       (LZH_TABLE -1)
#define         LZH_FREQ         0x8000

/**************************************************************************/
/**************************************************************************/
#ifndef BYTE
#define BYTE unsigned char
#endif
#ifndef WIN32
typedef struct { BYTE lowl, lowr, highl, highr; } offstruct;
#endif

class CHufComp
{
// Attributes
private:
#ifndef WIN32
	offstruct             sizeaddr;
#endif
	BYTE                  text_buf[LZH_NBUFS +LZH_LOOKA -1];
	short                 lson    [LZH_NBUFS +1],
		                  rson    [LZH_NBUFS +257],
			              dad     [LZH_NBUFS +1];

	unsigned long  textsize, codesize;
	unsigned short code, len, putbuf, getbuf;
	unsigned char  putlen, getlen;
	unsigned short freq[LZH_TABLE +1];        /* frequency table*/
	short          match_position,
                   match_length,
                   son [LZH_TABLE],           /* pointers to childnod.*/
                   prnt[LZH_TABLE +LZH_NCHAR];/* pointers to parentno.*/
                                                 /* except for T..T+NCHAR*/
                                                 /* which are used toget */
                                                 /* positions of leaves*/
                                                 /* corresponding tocodes*/


// Operations
public:
	int make_lzh_decomp(const char *inputfile, const char *outputfile, unsigned long MaxBytes = 0);
	int make_lzh_compre(const char *inputfile, const char *outputfile);

private:
	void lzh_insert_node(int r);
	void lzh_delete_node(int p);
	int lzh_get_bit(FILE *input);
	unsigned short lzh_get_byte(FILE *input);
	short lzh_put_code(FILE *output, int l, unsigned short c);
	void lzh_start_huff(void);
	void lzh_reconst(void);
	void lzh_update(int c);
	int lzh_encode_char(FILE *output, unsigned c);
	int lzh_encode_pos(FILE *output, unsigned short  c);
	int lzh_decode_char(FILE *input);
	int lzh_decode_pos(FILE *input);

// Implementation
public:
	CHufComp();
};


#endif // _COMPRESS_H

